# ===================================================
# ===================================================
# Functions to replicate Tables 1-4 in paper
# ===================================================
# ===================================================



# ===========================================================
# Wrapper function for calculations
# ===========================================================

# Inputs: 
# d: specifies the DGP
# AnzSim: number of Monte Carlo repetitions
# alpha: confidence level for risk measures
# n: sample size
# estimation: If "TRUE" ("FALSE"), GARCH parameters are (not) estimated
# cv.NA: critical values for normal approximation (e.g., 1.96 for 95%-c.i. for risk measures)
# v: number of initial fitlered residuals to discard before estimation


CI.95.optim.k <- function(d, AnzSim, alpha, n, estimation, cv.NA, v){
  res.H  <- matrix(0, nrow = 2 * length(alpha), ncol = 5)
  res.NP <- matrix(0, nrow = 2 * length(alpha), ncol = 2)
  
  for(i in 1:AnzSim){

    # Burr innovations
    if(d == 1){  
      cov    <- coverage.optim.k(GARCH.coef = c(0.95 * 20^2 / 252, 0.1, 0.85, 0), alpha, innov = "Burr", innov.param = c(0.25, 20), gamma=1/5, n, cv.NA, v, estimation, MIS = "No")  
    }
    if(d == 2){  
      cov    <- coverage.optim.k(GARCH.coef = c(0.95 * 20^2 / 252, 0.1, 0.85, 0), alpha, innov = "Burr", innov.param = c(1, 5), gamma=1/5, n, cv.NA, v, estimation, MIS = "No")  
    }
    
    if(d == 3){  
       cov    <- coverage.optim.k(GARCH.coef = c(0.95 * 20^2 / 252, 0.03, 0.09, 0.85, 0.00),  alpha, innov = "Burr", innov.param = c(0.25, 20), gamma=1/5, n, cv.NA, v, estimation, MIS = "Yes")  
    }               
    if(d == 4){  
       cov    <- coverage.optim.k(GARCH.coef = c(0.95 * 20^2 / 252, 0.03, 0.09, 0.85, 0.00),  alpha, innov = "Burr", innov.param = c(1, 5), gamma=1/5, n, cv.NA, v, estimation, MIS = "Yes")  
    }
     
     
    # Save Results 
    cov.H  <- cov$res.H
    res.H  <- res.H + cov.H
    
    cov.NP <- cov$res.NP
    res.NP <- res.NP + cov.NP
  }
  
  res.H  <- res.H  / AnzSim
  res.NP <- res.NP / AnzSim

  # Save results for CVaR and CExp
  res.H[ , 3]     <- sqrt( res.H[ , 3] )     # take root for RMSE
  dimnames(res.H) <- list( c(paste("CVaR H", alpha), paste("CExp H", alpha)), 
                             c("k*", "Bias", "RMSE", "Coverage", "Interval length") )
  
  res.NP[ , 2]     <- sqrt( res.NP[ , 2] )   # take root for RMSE
  dimnames(res.NP) <- list( c(paste("CVaR NP", alpha), paste("CExp NP", alpha)), 
                               c("Bias", "RMSE") )
  
  return(list(results.H = res.H, results.NP = res.NP))
}



# ===========================================================
# Function to calculate coverage etc. for one Monte Carlo repetition
# ===========================================================

# Inputs: 
# GARCH.coef: coefficients for GARCH model
# alpha: confidence level for risk measures
# innov: innovation distribution; one of "Burr", "Pareto", "t" (only "Burr" used in paper)
# innov.param: parameters for innovation distribution
# gamma: extreme value index of innovation distribution
# n: sample size
# cv.NA: critical values for normal approximation (e.g., 1.96 for 95%-c.i. for risk measures)
# v: number of initial fitlered residuals to discard before estimation
# estimation: If "TRUE" ("FALSE"), GARCH parameters are (not) estimated
# MIS: If "Yes" ("No"), misspecified (correctly specified) model is fitted

coverage.optim.k <- function(GARCH.coef, alpha, innov, innov.param, gamma, n, cv.NA, v, estimation, MIS){   # gamma=true extreme value index of innovations
   
   if(estimation == TRUE){
      
      cond=TRUE
      while (cond==TRUE){
         if(MIS == "No"){
            # 1. Simulate GARCH(1,1) model with Burr-distributed innovations 
            X <- garchsim_alt(GARCH.coef, alpha, innov, innov.param, n+v, ntrans = 1000)
         }
         if(MIS == "Yes"){
            X <- APARCHX.sim(GARCH.coef, delta0=1, alpha, innov, innov.param, n+v, ntrans = 1000)
         }
        
        
        # 2. Save true CVaR, CES and CExp 
        CExp.true <- X[[4]]
        CES.true  <- X[[3]]
        CVaR.true <- X[[2]]    
        X         <- X[[1]]   # for brevity
        
        
        # 3. Calculate estimated CVaR based on residuals U^
        if(gamma >= 1/4){   # if innovations have infinite 4th moments, use Laplace QMLE ...
          outp <- try(garchFit(formula = ~garch(1,1), data = X, init.rec = "mci", cond.dist = "ged",
                               shape = 1, include.mean = FALSE, include.shape = FALSE,
                               algorithm = "lbfgsb+nm", trace = FALSE) , silent = TRUE)
        }
        else{          # ... else use standard Gaussian QMLE
          outp <- try(garchFit(formula = ~garch(1,1), data = X, init.rec = "mci", cond.dist = "QMLE",
                               include.mean = FALSE, algorithm = "lbfgsb+nm", trace = FALSE) , silent = TRUE)
        }
        
        garch.coef = try(outp@fit$matcoef, silent=TRUE)
        
        omega1 = try(garch.coef[1], silent=TRUE)
        alpha1 = try(garch.coef[2], silent=TRUE)
        beta1  = try(garch.coef[3], silent=TRUE)
        
        if(is.na(omega1)){omega1=1}
        if(is.na(alpha1)){alpha1=1}
        if(is.na(beta1)){beta1=1}
        
        if(is.character(outp)){cond=TRUE}
        else{
          cond=((beta1>=1)|(omega1<0) | (alpha1<0) |(beta1<0)|((alpha1+beta1)>=1))
        }
        
        theta.hat <- c(omega1, alpha1, beta1)
      }
        X            <- X[(v+1) : (v+n)]            
        sigma.hat    <- outp@sigma.t[(v+1) : (v+n)]  # discard first v volatility estimates, due to initialization effects
        sigma.hat.n1 <- sqrt( theta.hat[1] + theta.hat[2] * X[n]^2 + theta.hat[3] * sigma.hat[n]^2 )   # volatility estimate in (n+1)
        U.hat        <- X / sigma.hat
   }
   else if(estimation == FALSE){
      # 1. Simulate GARCH(1,1) model with Burr-distributed innovations 
      X <- garchsim_alt(GARCH.coef, alpha, innov, innov.param, n+v, ntrans = 1000)
      
      # 2. Save true CVaR, CES and CExp 
      sigma.hat <- X[[5]]
      CExp.true <- X[[4]]
      CES.true  <- X[[3]]
      CVaR.true <- X[[2]]    
      X         <- X[[1]]   # for brevity
      
      sigma.hat    <- sigma.hat[(v+1) : (v+n)]
      X            <- X[(v+1) : (v+n)] 
      sigma.hat.n1 <- sqrt( GARCH.coef[1] + GARCH.coef[2] * X[n]^2 + GARCH.coef[3] * sigma.hat[n]^2 + GARCH.coef[4] * sigma.hat[n-1]^2 )   # volatility estimate in (n+1)
      U.hat        <- X / sigma.hat
   }
 
  
  # 3.1 Compute optimal k* using [Dea16+, p. 16] (Tail Index Estimation: Quantile Driven Threshold Selection)
  sort.U <- sort.int(U.hat, decreasing = TRUE, method = "quick")
  
  k.min  <- floor( log(n)^2 )    # minimal value of k
  k.max  <- floor( 4 * log(n)^2 )   # maximal value of k

  sup.H <- rep.int(NA, k.max)
  H     <- rep.int(NA, k.max)
  
  for(k in 1 : k.max){
    
    # 3.1.1 For CVaR estimation
    H[k]     <- 1/k * sum(log( sort.U[1 : (k+1)] / sort.U[(k+1)] ))     # Hill estimate based on (k+1) largest order statistics
    sup.H[k] <- max( abs(sort.U[1 : (k.max)] - (k/(1 : k.max))^( H[k] ) * sort.U[(k+1)]) )
  }
  k.star.H   <- which.min(sup.H[k.min : k.max]) + k.min - 1

  
  # 3.2 Calculate VaR, ES and expectile estimates using different k.star

  # 3.2.1 Calculate UNCONDITIONAL Risk measures
  H      <- H[k.star.H]
  H.EQ   <- sort.U[k.star.H+1] * ((n*(1-alpha)) / k.star.H)^(-H)
  H.Exp  <- H.EQ   * (1/H - 1)^(-H)

  
  # 3.2.2 Calculate CONDITIONAL Risk measures
  CVaR.H   <- sigma.hat.n1 * H.EQ   
  CExp.H   <- sigma.hat.n1 * H.Exp


  # 4. Calculate confidence intervals for CVaR, CES and CExp
  # 4.0 Calculate asymptotic variance of Hill estimator [Hil10, p.1412]
  # 4.0.1 Preliminaries
  gamma.n   <- max( 1, floor( k.star.H^0.25 ) )    # see http://www.unc.edu/~jbhill/tail_estim_GAUSS_v1.txt
  W.i       <- log( pmax.int(U.hat / sort.U[k.star.H+1], 1) ) - (k.star.H / n) * H
  
  # 4.0.2 Actual Calculation
  sigma.n.hat.sq <- sum(W.i^2)
  for(h in 1:gamma.n){       # Use Bartlett kernel max(0, 1-abs(x))
    Lag.h <- (1 - h / gamma.n) * sum( W.i[(h+1):n] * W.i[1:(n-h)] )
    sigma.n.hat.sq <- sigma.n.hat.sq + 2 * Lag.h
  }
  sigma.n.hat.sq <- (1 / k.star.H) * sigma.n.hat.sq
  
  
  # 4.1 Normal Approximation ...
  CI.H           <- CI.H.Exp       <- matrix(NA, nrow=length(alpha), ncol=2)
  coverage.H.VaR <- coverage.H.Exp <- rep.int(NA, length(alpha))
  VaR.NP  <-  Exp.NP <- rep.int(NA, length(alpha))
  CVaR.NP <- CExp.NP <- rep.int(NA, length(alpha))
  
  for(a in 1:length(alpha)){
     
     # ... for CVaR
     CI.H[a, 1]    <- CVaR.H[a] * exp(-cv.NA * sqrt(sigma.n.hat.sq) * abs(log(k.star.H / (n*(1-alpha[a])))) / sqrt(k.star.H) )
     CI.H[a, 2]    <- CVaR.H[a] * exp( cv.NA * sqrt(sigma.n.hat.sq) * abs(log(k.star.H / (n*(1-alpha[a])))) / sqrt(k.star.H) )
 
     # ... for CExp
     CI.H.Exp[a, 1] <- CExp.H[a] * exp(-cv.NA * sqrt(sigma.n.hat.sq) * abs(log(k.star.H / (n*(1-alpha[a])))) / sqrt(k.star.H) )
     CI.H.Exp[a, 2] <- CExp.H[a] * exp( cv.NA * sqrt(sigma.n.hat.sq) * abs(log(k.star.H / (n*(1-alpha[a])))) / sqrt(k.star.H) )
   
     
     # 5. Return if true CVaR / CES covered by c.i. ...
     # ... for CVaR
     coverage.H.VaR[a] <- (CI.H[a, 1]    < CVaR.true[a]) * (CI.H[a, 2]    > CVaR.true[a])   # = 1, iff  CVaR.true\in(CI.H[1], CI.up.H[2])

     # ... for CExp
     coverage.H.Exp[a] <- (CI.H.Exp[a, 1]< CExp.true[a]) * (CI.H.Exp[a, 2]> CExp.true[a])   # = 1, iff  CExp.true\in(CI.H.Exp[1], CI.H.Exp[2])
   
     
     # 6. Results for Non-Parametric (NP) Estimator
     
     # 6.1 NP-Quanile Estimator
     VaR.NP[a]  <- sort.int(U.hat, partial = floor(alpha[a] * n))[floor(alpha[a] * n)]
     CVaR.NP[a] <- sigma.hat.n1 * VaR.NP[a]
     
     # 6.2 NP-Expectile Estimator
     Exp.NP[a]  <- Expectile.est(U.hat, alpha[a])
     CExp.NP[a] <- sigma.hat.n1 * Exp.NP[a]
  }
  
  # 7. Collect Results
  # 7.1 For EVT Estimates
  res.VaR.H <- cbind( c(k.star.H, rep.int(NA, length(alpha)-1)), CVaR.H - CVaR.true, (CVaR.H - CVaR.true)^2, 
                           coverage.H.VaR, CI.H[ , 2] - CI.H[ , 1])
  res.Exp.H <- cbind( rep.int(NA, length(alpha)), CExp.H - CExp.true, (CExp.H - CExp.true)^2, 
                      coverage.H.Exp, CI.H.Exp[ , 2] - CI.H.Exp[ , 1])
  
  # 7.2 For NP Estimates
  res.VaR.NP <- cbind(CVaR.NP - CVaR.true, (CVaR.NP - CVaR.true)^2)
  res.Exp.NP <- cbind(CExp.NP - CExp.true, (CExp.NP - CExp.true)^2)
  
  return( list( res.H  = rbind( res.VaR.H, res.Exp.H ),
                res.NP = rbind( res.VaR.NP, res.Exp.NP ) ))
}



# ===========================================================
# Function to calculate non-parametric expectile estimate
# ===========================================================

# Inputs: 
# X: time series
# alpha: confidence level for expectile

Expectile.est <- function(X, alpha){    # LAWS estimator; see Daouia, Girard and Stupfler (2018, p. 8)
   loss.fct       <- function(y){ return( abs( alpha - 1*(y<=0) ) * y^2) }
   expectile.loss <- function(u){ return( sum(loss.fct(X-u)) ) }
   Exp.est        <- optimise(expectile.loss, interval = c(-100, 100), maximum = FALSE)$minimum
   return(Exp.est)
}




# ===========================================================
# Function to simulate GARCH processes with alternative innovation distributions
# ===========================================================

# Inputs: 
# GARCH.coef: coefficients for GARCH model
# alpha: confidence level for risk measures
# innov: innovation distribution; one of "Burr", "Pareto", "t" (only "Burr" used in paper)
# innov.param: parameters for innovation distribution
# n: sample size
# ntrans: length of burn-in period


garchsim_alt <- function(GARCH.coef, alpha, innov, innov.param, n, ntrans){
   VaR <- ES <- Expectile <- rep.int(NA, length(alpha))
   m   <- n + ntrans
  
  # 0. Calculate innovations eps
  # 0.1 For "Burr"-innovations
  
  if(identical(innov, "Burr")){
    lambda <- innov.param[1]
    tau    <- innov.param[2]
    sd.wt  <- sqrt(mburr(order=2, shape1 = lambda, shape2 = tau))
    eps    <- (2 * rbinom(m, 1, p=0.5) - 1) * rburr(m, shape1 = lambda, shape2 = tau) / sd.wt
    
    # 1 Calculate VaR and expectile of eps
    # 1.1 Define preliminary functions
    f.eps <- function(x){    # density function of the eps
      f.Burr.std <- 1/2 * dburr(abs(x) * sd.wt, shape1 = lambda, shape2 = tau) * sd.wt
      return(f.Burr.std)
    }
    q.burr <- function(x) return( qburr(2 * x - 1, shape1 = lambda, shape2 = tau) / sd.wt )  # quantile function of eps
    g <- function(x, alph) return(alph - G(x, f.eps))    # function for FOC
    f.eps.x <- function(x) return(f.eps(x)*x)       # function required to calculate ES
    
    # 1.2 Calculate the true values of the risk measures
    for(a in 1:length(alpha)){
       VaR[a]       <- q.burr(alpha[a])
       ES[a]        <- 1/(1-alpha[a]) * integrate(f.eps.x, lower=VaR[a], upper= Inf)$value
       Expectile[a] <- uniroot(g, interval=c(-2, 2), alph = alpha[a], extendInt = "yes")$root
    }
  }
  
  # 0.2 For "Pareto"-innovations
  else if(identical(innov, "Pareto")){
    shape   <- innov.param[1]
    minimum <- innov.param[2]
    sd.wt   <- sqrt(mpareto1(order=2, shape, minimum))
    eps     <- (2 * rbinom(m, 1, p=0.5) - 1) * rpareto1(m, shape, minimum) / sd.wt
    
    # 1. Calculate VaR, ES and expectile of eps
    # 1.1 Define preliminary functions
    f.eps <- function(x){    # density function of the eps
      f.Pareto.std <- 1/2 * dpareto1(abs(x) * sd.wt, shape, minimum) * sd.wt
      return(f.Pareto.std)
    }
    q.pareto <- function(x) return( qpareto1(2 * x - 1, shape, minimum) / sd.wt )  # quantile function of eps
    g <- function(x, alph) return(alph - G(x, f.eps))    # function for FOC
    f.eps.x <- function(x) return(f.eps(x)*x)       # function required to calculate ES
    
    # 1.2 Calculate the true values of the risk measures
    for(a in 1:length(alpha)){
       VaR[a]       <- q.pareto(alpha[a])
       ES[a]        <- 1/(1-alpha[a]) * integrate(f.eps.x, lower=VaR[a], upper= Inf)$value
       Expectile[a] <- uniroot(g, interval=c(-2, 2), alph = alpha[a], extendInt = "yes")$root
    }
  }
  
  # 0.3 For "t"-innovations
  else if(identical(innov, "t")){
    nu    <- innov.param[1]
    sd.wt <- sqrt( nu / (nu - 2) )
    eps   <- rt(m, df = nu) / sd.wt      # standardized t-distr. with E=0, Var=1
    
    # 1. Calculate VaR, ES and expectile of eps
    # 1.1 Define preliminary functions
    f.eps <- function(x){    # density function of the eps
      f.t.std <- dt(x * sd.wt, df = nu) * sd.wt
      return(f.t.std)
    }
    g <- function(x, alph) return(alph - G(x, f.eps))    # function for FOC
    f.eps.x <- function(x) return(f.eps(x)*x)       # function required to calculate ES
    
    # 1.2 Calculate the true values of the risk measures
    for(a in 1:length(alpha)){
       VaR[a]       <- qt(p=alpha[a], df = nu) / sd.wt
       ES[a]        <- 1/(1-alpha[a]) * integrate(f.eps.x, lower=VaR[a], upper= Inf)$value
       Expectile[a] <- uniroot(g, interval=c(-2, 2), alph=alpha[a], extendInt = "yes")$root
    }
  }
  
  # 1. GARCH(1,1) process
  X <- sigma.sq <- numeric(m)         # intializes vectors of y's and sigma^2's to be m long
  X[1:2]        <- rnorm(2)           # start value
  sigma.sq[1:2] <- rnorm(2)^2         # start value
  for(i in 3 : m){
    sigma.sq[i] <- GARCH.coef[1] + GARCH.coef[2] * X[i-1]^2 + GARCH.coef[3] * sigma.sq[i-1] + GARCH.coef[4] * sigma.sq[i-2]
    X[i]        <- sqrt(sigma.sq[i]) * eps[i]    # Simulate GARCH(1,1) process
  }

  # 2. Calcuate true CVaR and CES
  sigma.sq.np1 <- GARCH.coef[1] + GARCH.coef[2] * X[m]^2 + GARCH.coef[3] * sigma.sq[m] + GARCH.coef[4] * sigma.sq[m-1]
  CVaR.true    <- sqrt(sigma.sq.np1) * VaR
  CES.true     <- sqrt(sigma.sq.np1) * ES
  CExp.true    <- sqrt(sigma.sq.np1) * Expectile
  
  return( list(X[(ntrans + 1) : m], CVaR.true, CES.true, CExp.true, sqrt(sigma.sq[(ntrans + 1) : m])) )
}


# ===========================================================
# Function to simulate APARCH processes with alternative innovation distributions
# ===========================================================

# Inputs: 
# GARCH.coef: coefficients for GARCH model
# delta0: power of volatility to be modelled
# alpha: confidence level for risk measures
# innov: innovation distribution; one of "Burr", "Pareto", "t" (only "Burr" used in paper)
# innov.param: parameters for innovation distribution
# n: sample size
# ntrans: length of burn-in period


APARCHX.sim <- function(GARCH.coef, delta0, alpha, innov, innov.param, n, ntrans){ 
   VaR <- ES <- Expectile <- rep.int(NA, length(alpha))
   m   <- n + ntrans
   
   # 0. Calculate innovations eps
   # 0.1 For "Burr"-innovations
   
   if(identical(innov, "Burr")){
      lambda <- innov.param[1]
      tau    <- innov.param[2]
      sd.wt  <- sqrt(mburr(order=2, shape1 = lambda, shape2 = tau))
      eps    <- (2 * rbinom(m, 1, p=0.5) - 1) * rburr(m, shape1 = lambda, shape2 = tau) / sd.wt
      
      # 1 Calculate VaR and expectile of eps
      # 1.1 Define preliminary functions
      f.eps <- function(x){    # density function of the eps
         f.Burr.std <- 1/2 * dburr(abs(x) * sd.wt, shape1 = lambda, shape2 = tau) * sd.wt
         return(f.Burr.std)
      }
      q.burr <- function(x) return( qburr(2 * x - 1, shape1 = lambda, shape2 = tau) / sd.wt )  # quantile function of eps
      g <- function(x, alph) return(alph - G(x, f.eps))    # function for FOC
      f.eps.x <- function(x) return(f.eps(x)*x)       # function required to calculate ES
      
      # 1.2 Calculate the true values of the risk measures
      for(a in 1:length(alpha)){
         VaR[a]       <- q.burr(alpha[a])
         ES[a]        <- 1/(1-alpha[a]) * integrate(f.eps.x, lower=VaR[a], upper= Inf)$value
         Expectile[a] <- uniroot(g, interval=c(-2, 2), alph = alpha[a], extendInt = "yes")$root
      }
   }
   
   # 0.2 For "Pareto"-innovations
   else if(identical(innov, "Pareto")){
      shape   <- innov.param[1]
      minimum <- innov.param[2]
      sd.wt   <- sqrt(mpareto1(order=2, shape, minimum))
      eps     <- (2 * rbinom(m, 1, p=0.5) - 1) * rpareto1(m, shape, minimum) / sd.wt
      
      # 1. Calculate VaR, ES and expectile of eps
      # 1.1 Define preliminary functions
      f.eps <- function(x){    # density function of the eps
         f.Pareto.std <- 1/2 * dpareto1(abs(x) * sd.wt, shape, minimum) * sd.wt
         return(f.Pareto.std)
      }
      q.pareto <- function(x) return( qpareto1(2 * x - 1, shape, minimum) / sd.wt )  # quantile function of eps
      g <- function(x, alph) return(alph - G(x, f.eps))    # function for FOC
      f.eps.x <- function(x) return(f.eps(x)*x)       # function required to calculate ES
      
      # 1.2 Calculate the true values of the risk measures
      for(a in 1:length(alpha)){
         VaR[a]       <- q.pareto(alpha[a])
         ES[a]        <- 1/(1-alpha[a]) * integrate(f.eps.x, lower=VaR[a], upper= Inf)$value
         Expectile[a] <- uniroot(g, interval=c(-2, 2), alph = alpha[a], extendInt = "yes")$root
      }
   }
   
   # 0.3 For "t"-innovations
   else if(identical(innov, "t")){
      nu    <- innov.param[1]
      sd.wt <- sqrt( nu / (nu - 2) )
      eps   <- rt(m, df = nu) / sd.wt      # standardized t-distr. with E=0, Var=1
      
      # 1. Calculate VaR, ES and expectile of eps
      # 1.1 Define preliminary functions
      f.eps <- function(x){    # density function of the eps
         f.t.std <- dt(x * sd.wt, df = nu) * sd.wt
         return(f.t.std)
      }
      g <- function(x, alph) return(alph - G(x, f.eps))    # function for FOC
      f.eps.x <- function(x) return(f.eps(x)*x)       # function required to calculate ES
      
      # 1.2 Calculate the true values of the risk measures
      for(a in 1:length(alpha)){
         VaR[a]       <- qt(p=alpha[a], df = nu) / sd.wt
         ES[a]        <- 1/(1-alpha[a]) * integrate(f.eps.x, lower=VaR[a], upper= Inf)$value
         Expectile[a] <- uniroot(g, interval=c(-2, 2), alph=alpha[a], extendInt = "yes")$root
      }
   }
   
   # 1. GARCH(1,1) process
   X <- sigma.sqd <- numeric(m)         # intializes vectors of y's and sigma^2's to be m long
   x <- y         <- numeric(m)         # initialize covariate vector
   e <- rnorm(m)
   X[1] <- rnorm(1)                     # start value
   y[1] <- rnorm(1)                     # start value for exogenous covariate
   x[1] <- exp(y[1])
   sigma.sqd[1]   <- rnorm(1)^2         # start value
   for(i in 2 : m){
      y[i] <- 0.8 * y[i-1] + e[i]
      x[i] <- exp(y[i])
      sigma.sqd[i] <- GARCH.coef[1] +
         GARCH.coef[2] * max(X[i-1], 0)^(delta0)  + GARCH.coef[3] * max(-X[i-1], 0)^(delta0) +
         GARCH.coef[4] * sigma.sqd[i-1] +
         GARCH.coef[5] * x[i-1]    # exogenous covariates in variance equation
      X[i]        <- sigma.sqd[i]^(1/(delta0)) * eps[i]    # Simulate GARCH(1,1) process
   }
   
   # 2. Calcuate true CVaR, CES and CExp
   sigma.sqd.np1 <- GARCH.coef[1] +
      GARCH.coef[2] * max(X[m], 0)^(delta0) + GARCH.coef[3] * max(-X[m], 0)^(delta0) +
      GARCH.coef[4] * sigma.sqd[m] +
      GARCH.coef[5] * x[m]     # exogenous covariates in variance equation
   CVaR.true     <- sigma.sqd.np1^(1/delta0) * VaR
   CES.true      <- sigma.sqd.np1^(1/delta0) * ES
   CExp.true     <- sigma.sqd.np1^(1/delta0) * Expectile
   
   return( list(X[(ntrans + 1) : m], CVaR.true, CES.true, CExp.true, 
                sigma.sqd[(ntrans + 1) : m]) )
}



# ===========================================================
# Helper function to calculate true Expectile
# ===========================================================

# Inputs: 
# xi: Potential value for expectile
# f: Density of distribution for which expectile is sought

G <- function(xi, f){   # arguments: xi as potential value for expectile, f(.) is density of distribution for which expectile is sought
  integrand <- function(y) return(abs(y-xi) * f(y))  # we use FOC for minimisation problem in (4) to calculate expectile
  G.xi <- integrate(integrand, lower=-Inf, upper=xi)$value / integrate(integrand, lower=-Inf, upper=Inf)$value
  return(G.xi)
}
